@@ -1,7 +1,7 @@
 package Terminal::Identify;
 
 # Load the basic Perl pragmas.
-use 5.030000;
+use 5.0260000;
 use strict;
 use warnings;
 
@@ -21,125 +21,103 @@ our @EXPORT = qw(
 # Base class of this module.
 our @ISA = qw(Exporter);
 
-# Set the package version. 
-our $VERSION = '0.26';
+# Set the package version.
+our $VERSION = '0.27';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
 
-# Set the global output format.
-our $OutputFormat = ""; 
+# Define the directory for Inline.
+#use Inline Config => DIRECTORY => '/tmp/';
 
-# Set some file variables.
-our $FN_PASSWD;
-our $FN_SHELLS;
+# Use inline C code.
+#use Inline 'C';
 
 # Define the Perl BEGIN block.
 BEGIN {
     # Check the operating system.
     if ($^O eq "MSWin32") {
-        # Print message to terminal window.
+        # Print message into terminal window.
         print "This module works not on Windows. Bye.\n";
         # Exit script with error code 1.
         exit 1;
     };
-    # Set some variables.
-    $FN_PASSWD = "/etc/passwd";
-    $FN_SHELLS = "/etc/shells";
-    # Check if Linux command which exists.
-    if (! -x "/bin/which" && ! -x "/usr/bin/which") {
-        die "Linux command 'which' does not exist.\n";
-    };
-    # Check if grep, users and ps exists.
-    die unless `which grep`;
-    die unless `which users`;
-    die unless `which ps`;
-    # Check if passwd exists.
-    if (! -e $FN_PASSWD) {
-        die "$!, $FN_PASSWD not exists.\n";
-    };
-    # Check if shells exists.
-    if (! -e $FN_SHELLS) {
-        die "$!, $FN_SHELLS not exists.\n";
-    };
     # Set the subroutine aliases.
     *whichtermami = \&whichterminalami;
     *which_terminal = \&whichterminalami;
     *identify_terminal = \&whichterminalami;
 };
 
-# Define the directory for Inline. 
-use Inline Config => DIRECTORY => '/tmp/';
+# Check if Linux command which exists.
+if (! -x "/bin/which" && ! -x "/usr/bin/which") {
+    die "Linux command 'which' does not exist Bye.\n";
+    # Print message into terminal window.
+    # print "Linux command 'which' does not exist Bye.\n";
+    # Exit script with error code 2.
+    # exit 2;
+};
 
-# Use inline C code.
-use Inline 'C';
+# Check if grep exists.
+if (!defined `which grep 2>/dev/null` || `which grep 2>/dev/null` eq "") {
+    die "Linux command 'grep' does not exist Bye.\n";
+    # Print message into terminal window.
+    # print "Linux command 'grep' does not exist Bye.\n";
+    # Exit script with error code 3.
+    # exit 3;
+};
 
-# Assign terminal process name to known terminal name.
-our %termhash = (
-    # Successful tested.
-    "alacritty"              => "Alacritty",
-    "cool-retro-term"        => "Cool Retro Term",         
-    "deepin-terminal"        => "Deepin Terminal",         
-    "gnome-terminal-server"  => "Gnome Terminal",
-    "guake"                  => "Guake Terminal",        
-    "hyper"                  => "Hyper",                
-    "kitty"                  => "kitty",                
-    "konsole"                => "Konsole",              
-    "lilyterm"               => "LilyTerm",
-    "lxterminal"             => "LXTerminal",
-    "mate-terminal"          => "MATE-Terminal",
-    "mlterm"                 => "mlterm",
-    "mlterm-tiny"            => "mlterm-tiny",
-    "pterm"                  => "pterm",                
-    "qterminal"              => "QTerminal",            
-    "roxterm"                => "ROXTerm",
-    "rxvt"                   => "rxvt",                
-    "rxvt-unicode"           => "rxvt-unicode",
-    "sakura"                 => "Sakura",              
-    "screen"                 => "SCREEN",            
-    "stterm"                 => "stterm",            
-    "tabby"                  => "Tabby",
-    "terminator"             => "Terminator",         
-    "terminology"            => "Terminology",        
-    "termit"                 => "Termit",  
-    "tilda"                  => "Tilda", 
-    "tilix"                  => "Tilix",             
-    "tmux"                   => "tmux",         
-    "urxvt"                  => "urxvt",      
-    "uxterm"                 => "UXTerm",    
-    "xfce4-terminal"         => "Xfce4-Terminal-Emulator",
-    "xiterm+thai"            => "xiterm+thai",    
-    "xterm"                  => "XTerm",             
-    "wezterm"                => "WezTerm",
-    "yakuake"                => "Yakuake",   
-    # Known, but not tested. 
-    "altyo"                  => "AltYo",            
-    "aterm"                  => "aterm",                   
-    "black-screen"           => "Black Screen",    
-    "domterm"                => "DomTerm",      
-    "Eterm"                  => "Eterm",        
-    "extraterm"              => "Extraterm",           
-    "fbpad"                  => "Fbpad",               
-    "fbterm"                 => "Fbterm",              
-    "finalterm"              => "Final Term",          
-    "fish"                   => "FISH",                  
-    "flexterm"               => "FlexTerm",              
-    "foot"                   => "foot",                  
-    "havoc"                  => "Havoc",                 
-    "pantheon-terminal"      => "Pantheon Terminal",       
-    "putty"                  => "PuTTY",
-    "rxvt-unicode-truecolor" => "rxvt-unicode-truecolor",
-    "terminal"               => "Terminal",               
-    "terminix"               => "Terminix",             
-    "termite"                => "Termite",              
-    "tinyterm"               => "TinyTERM",             
-    "tym"                    => "tym",                     
-    "wazaterm"               => "Wazaterm",               
-    "wterm"                  => "Wterm",                  
-    "xiki"                   => "Xiki",                   
-    "xvt"                    => "Xvt"   
-);
+# Check if users exists.
+if (!defined `which users 2>/dev/null` || `which users 2>/dev/null` eq "") {
+    die "Linux command 'users' does not exist Bye.\n";
+    # Print message into terminal window.
+    # print "Linux command 'grep' does not exist Bye.\n";
+    # Exit script with error code 4.
+    # exit 4;
+};
 
+# Check if ps exists.
+if (!defined `which ps 2>/dev/null` || `which ps 2>/dev/null` eq "") {
+    die "Linux command 'ps' does not exist Bye.\n";
+    # Print message into terminal window.
+    # print "Linux command 'ps' does not exist Bye.\n";
+    # Exit script with error code 4.
+    # exit 5;
+};
+
+# Set some variables.
+our $FN_PASSWD = "/etc/passwd";
+our $FN_SHELLS = "/etc/shells";
+
+# Check if passwd exists.
+if (! -e $FN_PASSWD) {
+    die "$!, $FN_PASSWD not exists.\n";
+};
+# Check if shells exists.
+if (! -e $FN_SHELLS) {
+    die "$!, $FN_SHELLS not exists.\n";
+};
+
+# Set the global output format.
+our $OutputFormat = "";
+
+# Create the dictionary termhash.
+our %termhash = ();
+my $line = "";
+my $key = "";
+my $value = "";
+while (<DATA>) {
+    if (/# Begin terminals/../# End terminals/) {
+        next if /# Begin terminals/ || /# End terminals/;
+        $line = $_;
+        my @parts = split /=>/, $line, 2;
+        $key = $parts[0];
+        $key =~ s/^\s+|\s+$//g;
+        $value = $parts[1];
+        $value =~ s/^\s+|\s+$//g;
+        $termhash{$key} = $value;
+    };
+};
+    
 # ============================================================================ #
 # Subroutine trim                                                              #
 #                                                                              #
@@ -152,12 +130,12 @@ our %termhash = (
 # @return:   $string           Trimmed string  (scalar)                        #
 # ============================================================================ #
 sub trim {
-    # Assign the function argument to the local string variable $str.  
+    # Assign the function argument to the local string variable $str.
     my $string = ((defined $_[0] && $_[0] ne "") ? $_[0] : "");
     # Trim the string from the left side and the right side.
     $string =~ s/^\s+|\s+$//g;
     # Return the trimmed string.
-    return $string;    
+    return $string;
 };
 
 # ============================================================================ #
@@ -166,7 +144,7 @@ sub trim {
 # Description:                                                                 #
 # The subroutine adds the square brackets [ and ] to the searchstring for      #
 # use with grep. E.g. shell becomes [s]hell. Therefor the string must have     #
-# a minumum length of 1.                                                       # 
+# a minumum length of 1.                                                       #
 #                                                                              #
 # @argument: $_[0] => $str  Searchstring                       (scalar)        #
 # @return:   $str           Searchstring with square brackets  (scalar)        #
@@ -176,10 +154,10 @@ sub search_brackets {
     my $searchstring = "";
     # Check if $searchstring is defined and is not empty.
     if ((defined $_[0]) && (length($_[0]) > 0)) {
-        # Assign the subroutine argument to the local variable.  
+        # Assign the subroutine argument to the local variable.
         $searchstring = $_[0];
     } else {
-        # Return string of length 0. 
+        # Return string of length 0.
         return "";
     }
     # Add square brackets [ and ] to the given string.
@@ -200,7 +178,7 @@ sub search_brackets {
 # @return:   $content        File content  (scalar)                            #
 # ============================================================================ #
 sub read_file {
-    # Assign the function argument to the local variable.  
+    # Assign the function argument to the local variable.
     my $file = $_[0];
     # Open a file handler for reading.
     open(my $fh, "<", $file);
@@ -222,7 +200,7 @@ sub read_file {
 # @return:   $user_arr  Array with the logged-in users  (array)                #
 # ============================================================================ #
 sub login_users {
-    # Get the logged-in users from Linux command users.   
+    # Get the logged-in users from Linux command users.
     my $user = `users`;
     # Split the logged-in users and store them to the array.
     my @user_array = split ' ', $user;
@@ -254,7 +232,7 @@ sub login_shells {
     foreach (split '\n', $content) {
         # Trim the new line.
         my $line = trim($_);
-        # Use only a line with a valid login shell. 
+        # Use only a line with a valid login shell.
         if ($line =~ /^\/.*\/(.*)$/) {
             # Remove the path from the line with the login shell.
             my ($shell) = $line =~ /^\/.*\/(.*)$/;
@@ -273,21 +251,21 @@ sub login_shells {
 # Subroutine get_ppid                                                          #
 #                                                                              #
 # Description:                                                                 #
-# Determine the PPID of the calling Perl script using the Linux command ps.    # 
+# Determine the PPID of the calling Perl script using the Linux command ps.    #
 #                                                                              #
-# @argument: $_[0] => $pid  PID   (scalar)                                     # 
-# @return:   $ppid          PPID  (scalar)                                     # 
+# @argument: $_[0] => $pid  PID   (scalar)                                     #
+# @return:   $ppid          PPID  (scalar)                                     #
 # ============================================================================ #
 sub get_ppid {
     # Assign the subroutine argument to the local variable $pid.
-    my $pid = $_[0]; 
+    my $pid = $_[0];
     # Store the output of the Linux command ps in the local variable $ppid.
     my $ppid = `ps --no-headers -o ppid:1 -p $pid --sort lstart 2>/dev/null`;
     # Split a multiline Perl scalar with the PPID's in parts.
     my @parts = split /\s+/, $ppid;
     # If there is more than one PPID, use the first PPID.
     $ppid = $parts[0];
-    # Trim the Perl scalar with the PPID.  
+    # Trim the Perl scalar with the PPID.
     $ppid = trim($ppid);
     # Return the PPID.
     return $ppid;
@@ -332,9 +310,9 @@ sub get_username {
 # Description:                                                                 #
 # Create a multi dimensional array with term, user and shell.                  #
 #                                                                              #
-# @arguments: $_[0] => $passwd_content  Content of passwd      (scalar)        # 
-#             @{$_[1]} => @user_array   User array             (array)         # 
-#             @{$_[2]} => @shell_array  Shell array            (array)         # 
+# @arguments: $_[0] => $passwd_content  Content of passwd      (scalar)        #
+#             @{$_[1]} => @user_array   User array             (array)         #
+#             @{$_[2]} => @shell_array  Shell array            (array)         #
 # @return:    @result_array             Array with the result  (array)         #
 # ============================================================================ #
 sub term_user_shell {
@@ -357,10 +335,10 @@ sub term_user_shell {
     } else {
         # Check on pts and tty.
         if ($term_path =~ /^.*\/dev\/(pts\/\d+)$/) {
-            # Extract the pseudo terminal slave. 
+            # Extract the pseudo terminal slave.
             ($term) = $term_path =~ /^.*\/dev\/(pts\/\d+)$/;
         } elsif ($term_path =~ /^.*\/dev\/(tty\d+)$/) {
-            # Extract the terminal TeleTYpewriter. 
+            # Extract the terminal TeleTYpewriter.
             ($term) = $term_path =~ /^.*\/dev\/(tty\d+)$/;
         } else {
             $term = "?";
@@ -375,16 +353,16 @@ sub term_user_shell {
             # Check user and shell against the given arrays.
             if (grep(/^$user$/, @user_array) &&
                 grep(/^$shell$/, @shell_array)) {
-                # Assemble a new list.  
-                my @tmp = ($term, $user, $shell); 
+                # Assemble a new list.
+                my @tmp = ($term, $user, $shell);
                 # Add data to array.
                 push(@result_array, \@tmp);
             } elsif ($user eq $username) {
-                # Assemble a new list.  
-                my @tmp = ($term, $user, $shell); 
+                # Assemble a new list.
+                my @tmp = ($term, $user, $shell);
                 # Add data to array.
                 push(@result_array, \@tmp);
-            }; 
+            };
         };
     };
     # Return the array.
@@ -394,10 +372,10 @@ sub term_user_shell {
 # ============================================================================ #
 # Subroutine terminal_process                                                  #
 #                                                                              #
-# Description:                                                                 # 
+# Description:                                                                 #
 # Get the process related to term, user and shell.                             #
 #                                                                              #
-# @argument: @{$_[0]} => @data  Array with term, user and shell  (array)       # 
+# @argument: @{$_[0]} => @data  Array with term, user and shell  (array)       #
 # @return:   @result            Array with matching processes    (array)       #
 # ============================================================================ #
 sub terminal_process {
@@ -405,6 +383,8 @@ sub terminal_process {
     my @data = @{$_[0]};
     # Initialise the return array.
     my @match = ();
+    # Set command.
+    my $ps_cmd = "ps aux --sort lstart 2>/dev/null";
     # Loop over the elements of the array.
     foreach my $item (@data) {
         # Get term, user and shell from the array.
@@ -415,7 +395,10 @@ sub terminal_process {
         $user = search_brackets($user);
         $shell = search_brackets($shell);
         # Search for processes which is matching shell, user and terminal.
-        my $process = `ps aux --sort lstart 2>/dev/null | grep $shell | grep $user | grep $term`;
+        my $grep_shell = "grep $shell 2>/dev/null";
+        my $grep_user = "grep $user 2>/dev/null";
+        my $grep_term = "grep $term 2>/dev/null";
+        my $process = `${ps_cmd} | ${grep_shell} | ${grep_user} | ${grep_term}`;
         # Check if the result is not empty.
         if ($process ne "") {
             # Add the process to the return array.
@@ -433,7 +416,9 @@ sub terminal_process {
             $term = search_brackets($term);
             $user = search_brackets($user);
             # Search for processes which is matching shell and terminal.
-            my $process = `ps aux --sort lstart 2>/dev/null | grep $user | grep $term | grep "?" | grep "sshd"`;
+            my $grep_term = "grep $term 2>/dev/null";
+            my $grep_user = "grep $user 2>/dev/null";
+            my $process = `${ps_cmd} | ${grep_user} | ${grep_term} | grep "?" | grep "sshd"`;
             # Check if the result is not empty.
             if ($process ne "") {
                 # Add the process to the return array.
@@ -460,7 +445,7 @@ sub terminal_command_line {
     my $ppid = $_[0];
     # Get the command column from the ps output.
     my $termproc = `ps --no-headers -o cmd:1 -p $ppid --sort lstart 2>/dev/null`;
-    # Trim the command line output string.  
+    # Trim the command line output string.
     $termproc = trim($termproc);
     # Return the process related to the terminal in use.
     return $termproc
@@ -472,11 +457,11 @@ sub terminal_command_line {
 # Description:                                                                 #
 # Identify the process command line of the terminal in use.                    #
 #                                                                              #
-# @argument: None                                                              # 
-# @return:   $terminal  Process command line  (scalar)                         #
+# @argument: None                                                              #
+# @return:   $terminal_command  Process command line  (scalar)                 #
 # ============================================================================ #
 sub terminal_identifier {
-    # Declare the return variable $terminal.
+    # Declare the return variable $terminal_command.
     my $terminal_command;
     # Set the filename.
     my $filename = $FN_PASSWD;
@@ -490,7 +475,10 @@ sub terminal_identifier {
     my @result = term_user_shell($content, \@user_arr, \@shell_arr);
     # check if @result is empty.
     if (@result == 0) {
-        return "";
+        # Set terminal command.
+        $terminal_command = "";
+        # Return an empty string.
+        return $terminal_command;
     };
     # Create the array with user and term.
     my @match = terminal_process(\@result);
@@ -508,12 +496,12 @@ sub terminal_identifier {
         # Set terminal command.
         $terminal_command = "";
     };
-    # Return the terminal process command or NONE.
+    # Return the terminal process command.
     return $terminal_command;
 };
 
 # ============================================================================ #
-# Subroutine get_terminal_path                                                 # 
+# Subroutine get_terminal_path                                                 #
 # ============================================================================ #
 sub get_terminal_path {
     # Assign the subroutine argument to the local variable.
@@ -521,10 +509,10 @@ sub get_terminal_path {
     # Initialise the return variable.
     my $terminal_path = "";
     # Define the regular expression for white spaces.
-    my $re_ws = qr/\s+/; 
+    my $re_ws = qr/\s+/;
     # Define the regular expression for command line arguments.
     my $re_args = qr/(?<= )(-.*?)(?= |$)/;
-    # Check terminal command on white spaces. 
+    # Check terminal command on white spaces.
     if ($terminal_command =~ /$re_ws/) {
         # Initialise the progs array.
         my @progs = ();
@@ -536,7 +524,7 @@ sub get_terminal_path {
         my @parts = split /$re_ws/, $raw_string;
         # Loop over the elements of the array.
         foreach (@parts) {
-            # Check if the element is an executable. 
+            # Check if the element is an executable.
             if (`which $_` ne "") {
                 # Add executable to new array.
                 push(@progs, $_);
@@ -546,7 +534,7 @@ sub get_terminal_path {
         if (@progs == 0) {
             # Assign first element to path.
             $terminal_path = $parts[0];
-        } else { 
+        } else {
             if (@progs > 1) {
                 # Assign first element to path.
                 $terminal_path = $progs[1];
@@ -554,17 +542,17 @@ sub get_terminal_path {
                 # Assign second element to path.
                 $terminal_path = $progs[0];
             };
-        };  
+        };
     } else {
         # Set variable $terminal_path.
         $terminal_path = $terminal_command;
     };
     # Return the terminal name.
-    return $terminal_path; 
+    return $terminal_path;
 };
 
 # ============================================================================ #
-# Subroutine get_terminal_name                                                 # 
+# Subroutine get_terminal_name                                                 #
 # ============================================================================ #
 sub get_terminal_name {
     # Assign the subroutine argument to the local variable.
@@ -573,7 +561,7 @@ sub get_terminal_name {
     my $terminal_name = "";
     # Define the regular expression for program pathes.
     my $re_pp = qr/^\/.*\//;
-    # Check the variable $terminal_path on the existence of a path. 
+    # Check the variable $terminal_path on the existence of a path.
     if ($terminal_path =~ /$re_pp/) {
         # Remove the path from variable $terminal_path.
         $terminal_name = $terminal_path =~ s/$re_pp//r;
@@ -582,11 +570,11 @@ sub get_terminal_name {
         $terminal_name = $terminal_path;
     };
     # Return the terminal name.
-    return $terminal_name; 
+    return $terminal_name;
 };
 
 # ============================================================================ #
-# Subroutine get_terminal_ftn                                                  # 
+# Subroutine get_terminal_ftn                                                  #
 # ============================================================================ #
 sub get_terminal_ftn {
     # Assign the subroutine argument to the local variable.
@@ -595,23 +583,23 @@ sub get_terminal_ftn {
     my $terminal_ftn = "";
     # Check terminal name.
     if (defined $termhash{$terminal_name}) {
-        # Get the friendly terminal name. 
+        # Get the friendly terminal name.
         $terminal_ftn = $termhash{$terminal_name};
     } else {
-        # Set the friendly terminal name. 
+        # Set the friendly terminal name.
         $terminal_ftn = $terminal_name;
     };
     # Return the friendly terminal name.
-    return $terminal_ftn; 
+    return $terminal_ftn;
 };
 
 # ============================================================================ #
-# Subroutine whichterminalami                                                  # 
+# Subroutine whichterminalami                                                  #
 #                                                                              #
 # Description:                                                                 #
 # Identify the terminal emulator.                                              #
 #                                                                              #
-# @argument: $_[0] => $flag  Output format flag  (scalar)                      # 
+# @argument: $_[0] => $flag  Output format flag  (scalar)                      #
 # @return:   $terminal       Terminal name       (scalar)                      #
 # ============================================================================ #
 sub whichterminalami {
@@ -666,21 +654,139 @@ sub whichterminalami {
 1;
 
 __DATA__
-__C__
+
+# /etc/terminals: valid terminal emulators
+# Begin terminals
+alacritty               => Alacritty
+altyo                   => AltYo 
+aminal                  => Aminal 
+aterm                   => aterm 
+blackbox                => Black Box
+black-screen            => Black Screen 
+boxi                    => Boxi 
+contour                 => Contour Terminal 
+cool-retro-term         => Cool Retro Term 
+deepin-terminal         => Deepin Terminal 
+domterm                 => DomTerm 
+edex-ui                 => eDEX-UI 
+Eterm                   => Eterm 
+extraterm               => Extraterm 
+fbpad                   => Fbpad 
+fbterm                  => Fbterm 
+finalterm               => Final Term 
+flexterm                => Flex Term 
+foot                    => foot 
+gnome-terminal-server   => Gnome Terminal 
+guake                   => Guake Terminal 
+havoc                   => Havoc 
+hyper                   => Hyper 
+iterm2                  => iTerm2 
+kitty                   => kitty 
+konsole                 => Konsole 
+kterm                   => kterm 
+lilyterm                => LilyTerm 
+lxterminal              => LXTerminal 
+macterm                 => MacTerm 
+macwise                 => MacWise 
+mate-terminal           => MATE-Terminal 
+mlterm                  => mlterm 
+mlterm-tiny             => mlterm-tiny 
+mrxvt                   => mrxvt 
+okidk                   => OkiDK 
+pantheon-terminal       => Pantheon Terminal 
+pterm                   => pterm
+powershell              => PowerShell 
+putty                   => PuTTY 
+qterminal               => QTerminal
+rocket                  => Rocket 
+roxterm                 => ROXTerm 
+rxvt                    => rxvt 
+rxvt-unicode            => rxvt-unicode 
+rxvt-unicode-truecolor  => rxvt-unicode-truecolor 
+sakura                  => Sakura 
+screen                  => SCREEN 
+stterm                  => stterm 
+tabby                   => Tabby 
+terminal                => Terminal
+terminalpp              => terminalpp 
+terminator              => Terminator 
+terminix                => Terminix 
+terminology             => Terminology
+terminus                => Terminus 
+termite                 => Termite 
+termit                  => Termit 
+tilda                   => Tilda 
+tilix                   => Tilix 
+tinyterm                => TinyTERM 
+tmux                    => tmux 
+tterm                   => TTerm 
+tym                     => tym 
+upterm                  => Upterm 
+urxvt                   => urxvt 
+uxterm                  => UXTerm 
+warp                    => Warp
+wazaterm                => Wazaterm 
+wezterm                 => WezTerm 
+windterm                => WindTerm
+wterm                   => Wterm 
+xfce4-terminal          => Xfce4 Terminal 
+xiki                    => Xiki 
+xiterm+thai             => xiterm+thai 
+xterm                   => XTerm 
+xvt                     => Xvt 
+yakuake                 => Yakuake 
+zoc                     => ZOC
+zterm                   => ZTerm
+# End terminals
+# EOF
+
+# /etc/shells: valid login shells
+# Begin shells
+/usr/bin/ash
+/usr/bin/bash
+/usr/bin/csh
+/usr/bin/dash
+/usr/bin/dtksh
+/usr/bin/es
+/usr/bin/esh
+/usr/bin/fish
+/usr/bin/jsh
+/usr/bin/ksh
+/usr/bin/mksh
+/usr/bin/nash
+/usr/bin/oksh
+/usr/bin/pdksh
+/usr/bin/pfcsh
+/usr/bin/pfksh
+/usr/bin/psh
+/usr/bin/psh2
+/usr/bin/rbash
+/usr/bin/rc
+/usr/bin/sash
+/usr/bin/screen
+/usr/bin/scsh
+/usr/bin/sh
+/usr/bin/tcsh
+/usr/bin/tmux
+/usr/bin/zsh
+# End shells
+# EOF
+
+#__C__
 
 #include <stdio.h>
 #include <unistd.h>
 
-/*
- * -------------------
- * Function TermPath()
- * -------------------
- */
-char* TermPath() {
-  char* tty_name = ttyname(STDIN_FILENO);
-  char* term_path = (tty_name != NULL) ? tty_name : NULL;
-  return term_path;
-};
+#/*
+# * -------------------
+# * Function TermPath()
+# * -------------------
+# */
+#char* TermPath() {
+#  char* tty_name = ttyname(STDIN_FILENO);
+#  char* term_path = (tty_name != NULL) ? tty_name : NULL;
+#  return term_path;
+#};
 
 __END__
 
@@ -737,7 +843,7 @@ terminal.
   $OutputFormat
 
 If the global variable is set, the method arguments are ignored if existing in
-the method call. Unset the process name of the terminal emulator is printed to 
+the method call. Unset the process name of the terminal emulator is printed to
 the terminal window.
 
 =head2 Output format
@@ -747,7 +853,7 @@ by the options C<"PROC">, C<"PATH"> and C<"FTN">.
 
 The output format is as follows:
 
-=over 4 
+=over 4
 
   PROC => Full process command line of the terminal emulator
 
@@ -762,25 +868,28 @@ The output format is as follows:
 The main objective of this package is to provide a method which is capable of
 identifying the terminal emulator a logged-in user is actual using. In addition
 to the terminal emulator, the system console and a remote console are also
-recognised. The logged-in user is related to a valid login shell directly. The
-login shell of the logged-in user as well as the logged-in user is determined.
-Next the terminal path to the pseudo terminal slave (pts) is identified. Based
-on the previously informations the related process of the logged-in user,
-the login shell and the terminal path is determined. The evaluation of the PID
-of the process of the current running Perl script results in the PPID. The
-command related to this PPID is the name of the terminal emulator in use. The
-package works together with different terminal emulators. When terminal
-emulators are spawned from an initial invoked terminal emulator, each terminal
-emulator is correctly recognised. If the logged-in user changes during the
-session, this is recognised. Also using the sudo command does not affect the
-recognition of the terminal emulator. The terminal emulator in use by the
-logged-in user can be identified by the main command C<whichterminalami()> or
-the other defined aliases.
+recognised.
+
+The logged-in user is related to a valid login shell directly. The login shell
+of the logged-in user as well as the logged-in user is determined. Next the
+terminal path to the pseudo terminal slave (pts) is identified. Based on the
+previously informations the related process of the logged-in user, the login
+shell and the terminal path is determined. The evaluation of the PID of the
+process of the current running Perl script results in the PPID. The command
+related to this PPID is the name of the terminal emulator in use. The package
+works together with different terminal emulators. When terminal emulators are
+spawned from an initial invoked terminal emulator, each terminal emulator is
+correctly recognised. If the logged-in user changes during the session, this
+is recognised. Also using the sudo command does not affect the recognition of
+the terminal emulator.
+
+The terminal emulator in use by the logged-in user can be identified by the
+main command C<whichterminalami()> or the other defined aliases.
 
 =head1 EXAMPLES
 
 =head2 Example 1
-  
+
   # Load the Perl module.
   use Terminal::Identify;
 
@@ -796,7 +905,7 @@ the other defined aliases.
   print $terminal . "\n";
 
   # Method call with argument "PATH".
-  $terminal = whichterminalami("PATH"); 
+  $terminal = whichterminalami("PATH");
   print $terminal . "\n";
 
   # Method call with argument "FTN".
@@ -804,7 +913,7 @@ the other defined aliases.
   print $terminal . "\n";
 
 =head2 Example 2
-  
+
   # Load the Perl module.
   use Terminal::Identify;
 
@@ -848,7 +957,7 @@ operating systems in general until something else was shown.
 
 The following Linux commands must be available for package functionality:
 
-=over 4 
+=over 4
 
 =item * ps
 
@@ -862,7 +971,7 @@ The following Linux commands must be available for package functionality:
 
 The subsequent system files must be exist for package functionality:
 
-=over 4 
+=over 4
 
 =item * /etc/shells
 
@@ -888,6 +997,8 @@ Terminal emulators tested so far with the package:
 
 =item * Aterm
 
+=item * Black Box
+
 =item * Cool Retro Term
 
 =item * Deepin Terminal
@@ -922,7 +1033,7 @@ Terminal emulators tested so far with the package:
 
 =item * Sakura
 
-=item * SCREEN
+=item * screen
 
 =item * Tabby
 
@@ -946,10 +1057,17 @@ Terminal emulators tested so far with the package:
 
 =item * Xterm
 
-=item * Yakuake 
+=item * Yakuake
 
 =back
 
+=head1 TESTING ENVIRONMENT
+
+The package has been installed on various personal computers and laptops. This
+was done in the desktop environment using different terminal emulators, logged
+into the system console locally and logged in remotely. The Linux OS is based
+on Debian.
+
 =head1 LIMITATIONS
 
 The limitations of the package are given by the Linux commands and the Linux
@@ -958,22 +1076,28 @@ command C<users>, the Linux command C<which> and the Linux command C<grep> must
 be available. The Linux system files C</etc/shells> and C</etc/passwd> must be
 exist.
 
-When the Linux command su is used, then the detection results in su as terminal 
+When the Linux command su is used, then the detection results in su as terminal
 emulator, which is wrong. This has to be checked and changed.
 
 =head1 OPEN ISSUES
 
-Check if operating system is Linux, Unix, Unix-like, MacOS or Windows. Disallow 
-start on Windows, based on the fact, that the package will not work on Windows.
+Terminal emulators which are installed using Flatpak are not all the time correct
+identified.
 
 =head1 KNOWN BUGS
 
-The terminal emulator WezTerm can not be identified. Reason is a missing pseudo
-terminal slave while analysing the connected terminal window.
+The system file /etc/shells is not existing on solaris. If /etc/shells not exists,
+a backup solution must be integrated. This is a to-do for one of the next versions.
 
 =head1 ERROR CODES
 
-The package returns no error codes at the moment.
+  1 = Windows is recognised as OS.
+  * 2 = Linux command 'which' does not exist.
+  * 3 = Linux command 'grep' does not exist.
+  * 4 = Linux command 'users' does not exist.
+  * 5 = Linux command 'ps' does not exist.
+
+* not used yet. will be used after several tests on several OS.
 
 =head1 NOTES
 
@@ -985,7 +1109,7 @@ instead of the C code.
 
 The Linux command ps can be used e.g. in Bash to find out, which terminal
 emulator is in use by the current user. The command line to do this is quite
-easy: 
+easy:
 
   ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$)
 
@@ -998,7 +1122,7 @@ This can be verified by following command line:
 
 As consequence of this statement the PPID is the PID of the terminal emulator.
 The command related to the PPID is the name or process which invoked the terminal
-emulator. 
+emulator.
 
 If this procedure is done from within a script it fails in recognition of the
 terminal emulator. There are a few hurdles to overcome in order to carry out this
@@ -1010,7 +1134,7 @@ emulator.
 
 When the user logs in into the desktop, the user is connected to a login shell.
 Nevertheless, the user, which executes a script, can be different to the
-logged-in user. 
+logged-in user.
 
 Whenever a terminal is opened, this terminal is connected to a pseudo terminal
 slave. These pseudo-terminal slaves are numbered consecutively.
